@@ -18,7 +18,7 @@ import sbi.utils as utils
 from sbi.inference import NeuralInference
 from sbi.inference.posterior import NeuralPosterior
 from sbi.types import OneOrMore, ScalarFloat
-from sbi.utils import handle_invalid_x, warn_on_invalid_x, x_shape_from_simulation
+from sbi.utils import check_estimator_arg, x_shape_from_simulation
 
 
 class LikelihoodEstimator(NeuralInference, ABC):
@@ -70,6 +70,7 @@ class LikelihoodEstimator(NeuralInference, ABC):
         # `_build_neural_net`. It will be called in the first round and receive
         # thetas and xs as inputs, so that they can be used for shape inference and
         # potentially for z-scoring.
+        check_estimator_arg(density_estimator)
         if isinstance(density_estimator, str):
             self._build_neural_net = utils.likelihood_nn(model=density_estimator)
         else:
@@ -119,17 +120,13 @@ class LikelihoodEstimator(NeuralInference, ABC):
 
         for round_, num_sims in enumerate(num_sims_per_round):
 
-            # Generate parameters theta from prior in first round, and from most recent
-            # posterior estimate in subsequent rounds.
-            if round_ == 0:
-                theta = self._prior.sample((num_sims,))
-            else:
-                theta = self._posterior.sample(
-                    (num_sims,), show_progress_bars=self._show_progress_bars
-                )
+            # Run simulations for the round.
+            theta, x = self._run_simulations(round_, num_sims)
 
-            x = self._batched_simulator(theta)
-            x_shape = x_shape_from_simulation(x)
+            self._append_to_data_bank(theta, x, round_)
+
+            # Load data from most recent round.
+            theta, x, _ = self._get_from_data_bank(round_, exclude_invalid_x, False)
 
             # First round or if retraining from scratch:
             # Call the `self._build_neural_net` with the rounds' thetas and xs as
@@ -137,6 +134,7 @@ class LikelihoodEstimator(NeuralInference, ABC):
             # This is passed into NeuralPosterior, to create a neural posterior which
             # can `sample()` and `log_prob()`. The network is accessible via `.net`.
             if round_ == 0 or retrain_from_scratch_each_round:
+                x_shape = x_shape_from_simulation(x)
                 self._posterior = NeuralPosterior(
                     method_family="snle",
                     neural_net=self._build_neural_net(theta, x),
@@ -146,15 +144,7 @@ class LikelihoodEstimator(NeuralInference, ABC):
                     mcmc_method=self._mcmc_method,
                     get_potential_function=PotentialFunctionProvider(),
                 )
-            self._handle_x_o_wrt_amortization(x_o, x_shape, num_rounds)
-
-            # Check for NaNs in simulations.
-            is_valid_x, num_nans, num_infs = handle_invalid_x(x, exclude_invalid_x)
-            warn_on_invalid_x(num_nans, num_infs, exclude_invalid_x)
-
-            # Store (theta, x) pairs.
-            self._theta_bank.append(theta[is_valid_x])
-            self._x_bank.append(x[is_valid_x])
+                self._handle_x_o_wrt_amortization(x_o, x_shape, num_rounds)
 
             # Fit neural likelihood to newly aggregated dataset.
             self._train(
@@ -165,6 +155,7 @@ class LikelihoodEstimator(NeuralInference, ABC):
                 stop_after_epochs=stop_after_epochs,
                 max_num_epochs=max_num_epochs,
                 clip_max_norm=clip_max_norm,
+                exclude_invalid_x=exclude_invalid_x,
                 discard_prior_samples=discard_prior_samples,
             )
 
@@ -176,8 +167,8 @@ class LikelihoodEstimator(NeuralInference, ABC):
             self._summarize(
                 round_=round_,
                 x_o=self._posterior.default_x,
-                theta_bank=self._theta_bank,
-                x_bank=self._x_bank,
+                theta_bank=theta,
+                x_bank=x,
             )
 
         self._posterior._num_trained_rounds = num_rounds
@@ -192,6 +183,7 @@ class LikelihoodEstimator(NeuralInference, ABC):
         stop_after_epochs: int,
         max_num_epochs: int,
         clip_max_norm: Optional[float],
+        exclude_invalid_x: bool,
         discard_prior_samples: bool,
     ) -> None:
         r"""
@@ -206,8 +198,10 @@ class LikelihoodEstimator(NeuralInference, ABC):
 
         # Starting index for the training set (1 = discard round-0 samples).
         start_idx = int(discard_prior_samples and round_ > 0)
+        theta, x, _ = self._get_from_data_bank(start_idx, exclude_invalid_x)
+
         # Get total number of training examples.
-        num_examples = sum(len(theta) for theta in self._theta_bank)
+        num_examples = len(theta)
 
         # Select random train and validation splits from (theta, x) pairs.
         permuted_indices = torch.randperm(num_examples)
@@ -219,9 +213,7 @@ class LikelihoodEstimator(NeuralInference, ABC):
         )
 
         # Dataset is shared for training and validation loaders.
-        dataset = data.TensorDataset(
-            torch.cat(self._x_bank[start_idx:]), torch.cat(self._theta_bank[start_idx:])
-        )
+        dataset = data.TensorDataset(x, theta)
 
         # Create neural net and validation loaders using a subset sampler.
         train_loader = data.DataLoader(
