@@ -5,12 +5,13 @@ from __future__ import annotations
 
 from abc import ABC, abstractmethod
 from copy import deepcopy
+from sbi.user_input.user_input_checks import check_estimator_arg
 from typing import Callable, Dict, Optional, Tuple, Union, cast
 from warnings import warn
 
 import numpy as np
 import torch
-from torch import Tensor, nn, ones, optim, zeros
+from torch import Tensor, nn, ones, optim
 from torch.nn.utils import clip_grad_norm_
 from torch.utils import data
 from torch.utils.data.sampler import SubsetRandomSampler
@@ -20,7 +21,7 @@ import sbi.utils as utils
 from sbi.inference import NeuralInference
 from sbi.inference.posterior import NeuralPosterior
 from sbi.types import OneOrMore, ScalarFloat
-from sbi.utils import handle_invalid_x, warn_on_invalid_x, x_shape_from_simulation
+from sbi.utils import check_estimator_arg, x_shape_from_simulation
 
 
 class PosteriorEstimator(NeuralInference, ABC):
@@ -75,6 +76,7 @@ class PosteriorEstimator(NeuralInference, ABC):
         # `_build_neural_net`. It will be called in the first round and receive
         # thetas and xs as inputs, so that they can be used for shape inference and
         # potentially for z-scoring.
+        check_estimator_arg(density_estimator)
         if isinstance(density_estimator, str):
             self._build_neural_net = utils.posterior_nn(model=density_estimator)
         else:
@@ -83,7 +85,7 @@ class PosteriorEstimator(NeuralInference, ABC):
         self._sample_with_mcmc = sample_with_mcmc
         self._mcmc_method = mcmc_method
 
-        self._prior_masks, self._model_bank = [], []
+        self._model_bank = []
 
         # Extra SNPE-specific fields summary_writer.
         self._summary.update({"rejection_sampling_acceptance_rates": []})  # type:ignore
@@ -158,15 +160,19 @@ class PosteriorEstimator(NeuralInference, ABC):
         for round_, num_sims in enumerate(num_sims_per_round):
 
             # Run simulations for the round.
-            theta, x, prior_mask = self._run_simulations(round_, num_sims)
-            x_shape = x_shape_from_simulation(x)
+            theta, x = self._run_simulations(round_, num_sims)
+            self._append_to_data_bank(theta, x, round_)
+
+            # Load data from most recent round.
+            theta, x, _ = self._get_from_data_bank(round_, exclude_invalid_x, False)
 
             # First round or if retraining from scratch:
             # Call the `self._build_neural_net` with the rounds' thetas and xs as
-            # arguments, which will build the neural network
+            # arguments, which will build the neural network.
             # This is passed into NeuralPosterior, to create a neural posterior which
             # can `sample()` and `log_prob()`. The network is accessible via `.net`.
             if round_ == 0 or retrain_from_scratch_each_round:
+                x_shape = x_shape_from_simulation(x)
                 self._posterior = NeuralPosterior(
                     method_family="snpe",
                     neural_net=self._build_neural_net(theta, x),
@@ -176,16 +182,7 @@ class PosteriorEstimator(NeuralInference, ABC):
                     mcmc_method=self._mcmc_method,
                     get_potential_function=PotentialFunctionProvider(),
                 )
-            self._handle_x_o_wrt_amortization(x_o, x_shape, num_rounds)
-
-            # Check for NaNs in simulations.
-            is_valid_x, num_nans, num_infs = handle_invalid_x(x, exclude_invalid_x)
-            warn_on_invalid_x(num_nans, num_infs, exclude_invalid_x)
-
-            # XXX Rename bank -> rounds/roundwise.
-            self._theta_bank.append(theta[is_valid_x])
-            self._x_bank.append(x[is_valid_x])
-            self._prior_masks.append(prior_mask[is_valid_x])
+                self._handle_x_o_wrt_amortization(x_o, x_shape, num_rounds)
 
             # Fit posterior using newly aggregated data set.
             self._train(
@@ -197,6 +194,7 @@ class PosteriorEstimator(NeuralInference, ABC):
                 max_num_epochs=cast(int, max_num_epochs),
                 clip_max_norm=clip_max_norm,
                 calibration_kernel=calibration_kernel,
+                exclude_invalid_x=exclude_invalid_x,
                 discard_prior_samples=discard_prior_samples,
             )
 
@@ -222,8 +220,8 @@ class PosteriorEstimator(NeuralInference, ABC):
             self._summarize(
                 round_=round_,
                 x_o=self._posterior.default_x,
-                theta_bank=self._theta_bank,
-                x_bank=self._x_bank,
+                theta_bank=theta,
+                x_bank=x,
                 posterior_samples_acceptance_rate=acceptance_rate,
             )
 
@@ -240,50 +238,6 @@ class PosteriorEstimator(NeuralInference, ABC):
     ) -> Tensor:
         raise NotImplementedError
 
-    def _run_simulations(
-        self, round_: int, num_sims: int,
-    ) -> Tuple[Tensor, Tensor, Tensor]:
-        r"""
-        Run the simulations for a given round.
-
-        Args:
-            round_: Round number.
-            num_sims: Number of desired simulations for the round.
-
-        Returns:
-            theta: Parameters used for training.
-            x: Simulations used for training.
-            prior_mask: Whether each simulation came from a prior parameter sample.
-        """
-
-        if round_ == 0:
-            theta = self._prior.sample((num_sims,))
-
-            x = self._batched_simulator(theta)
-        else:
-            theta = self._posterior.sample(
-                (num_sims,),
-                x=self._posterior.default_x,
-                show_progress_bars=self._show_progress_bars,
-            )
-
-            x = self._batched_simulator(theta)
-
-        return theta, x, self._mask_sims_from_prior(round_, theta.size(0))
-
-    def _mask_sims_from_prior(self, round_: int, num_simulations: int) -> Tensor:
-        """Returns Tensor True where simulated from prior parameters.
-
-        Args:
-            round_: Current training round, starting at 0.
-            num_simulations: Actually performed simulations. This number can be below
-                the one fixed for the round if leakage correction through sampling is
-                active and `patience` is not enough to reach it.
-        """
-
-        prior_mask_values = ones if round_ == 0 else zeros
-        return prior_mask_values((num_simulations, 1), dtype=torch.bool)
-
     def _train(
         self,
         round_: int,
@@ -294,6 +248,7 @@ class PosteriorEstimator(NeuralInference, ABC):
         max_num_epochs: int,
         clip_max_norm: Optional[float],
         calibration_kernel: Callable,
+        exclude_invalid_x: bool,
         discard_prior_samples: bool,
     ) -> None:
         r"""Train the conditional density estimator for the posterior $p(\theta|x)$.
@@ -307,9 +262,10 @@ class PosteriorEstimator(NeuralInference, ABC):
 
         # Starting index for the training set (1 = discard round-0 samples).
         start_idx = int(discard_prior_samples and round_ > 0)
-        num_total_examples = sum(len(theta) for theta in self._theta_bank[start_idx:])
+        theta, x, prior_masks = self._get_from_data_bank(start_idx, exclude_invalid_x)
 
         # Select random neural net and validation splits from (theta, x) pairs.
+        num_total_examples = len(theta)
         permuted_indices = torch.randperm(num_total_examples)
         num_training_examples = int((1 - validation_fraction) * num_total_examples)
         num_validation_examples = num_total_examples - num_training_examples
@@ -319,11 +275,7 @@ class PosteriorEstimator(NeuralInference, ABC):
         )
 
         # Dataset is shared for training and validation loaders.
-        dataset = data.TensorDataset(
-            torch.cat(self._theta_bank[start_idx:]),
-            torch.cat(self._x_bank[start_idx:]),
-            torch.cat(self._prior_masks[start_idx:]),
-        )
+        dataset = data.TensorDataset(theta, x, prior_masks)
 
         # Create neural net and validation loaders using a subset sampler.
         train_loader = data.DataLoader(
