@@ -47,7 +47,7 @@ def test_api_sre_on_linearGaussian(num_dim: int):
 
     posterior = infer(num_rounds=1, num_simulations_per_round=1000, max_num_epochs=5)
 
-    posterior.sample(sample_shape=(10,), x=x_o, num_chains=2)
+    posterior.sample(sample_shape=(10,), x=x_o, mcmc_parameters={"num_chains": 2})
 
 
 def test_c2st_sre_on_linearGaussian_different_dims(set_seed):
@@ -101,7 +101,57 @@ def test_c2st_sre_on_linearGaussian_different_dims(set_seed):
     )
 
     posterior = infer(num_rounds=1, num_simulations_per_round=5000)
-    samples = posterior.sample((num_samples,), x=x_o, thin=3)
+    samples = posterior.sample((num_samples,), x=x_o, mcmc_parameters = {"thin": 3})
+
+    # Compute the c2st and assert it is near chance level of 0.5.
+    check_c2st(samples, target_samples, alg="snpe_c")
+
+
+@pytest.mark.slow
+def test_c2st_sre_external_data_on_linearGaussian(set_seed):
+    """Test whether SNPE C infers well a simple example with available ground truth.
+
+    Args:
+        set_seed: fixture for manual seeding
+    """
+    num_dim = 2
+
+    device = "cpu"
+    configure_default_device(device)
+    x_o = zeros(1, num_dim)
+    num_samples = 1000
+
+    # likelihood_mean will be likelihood_shift+theta
+    likelihood_shift = -1.0 * ones(num_dim)
+    likelihood_cov = 0.3 * eye(num_dim)
+
+    prior_mean = zeros(num_dim)
+    prior_cov = eye(num_dim)
+    prior = MultivariateNormal(loc=prior_mean, covariance_matrix=prior_cov)
+    gt_posterior = true_posterior_linear_gaussian_mvn_prior(
+        x_o[0], likelihood_shift, likelihood_cov, prior_mean, prior_cov
+    )
+    target_samples = gt_posterior.sample((num_samples,))
+
+    def simulator(theta):
+        return linear_gaussian(theta, likelihood_shift, likelihood_cov)
+
+    infer = SRE(
+        *prepare_for_sbi(simulator, prior),
+        simulation_batch_size=1000,
+        show_progress_bars=False,
+        device=device,
+    )
+
+    external_theta = prior.sample((1000,))
+    external_x = simulator(external_theta)
+
+    infer.provide_presimulated(external_theta, external_x)
+
+    posterior = infer(
+        num_rounds=1, num_simulations_per_round=1000, training_batch_size=100,
+    ).set_default_x(x_o)
+    samples = posterior.sample((num_samples,))
 
     # Compute the c2st and assert it is near chance level of 0.5.
     check_c2st(samples, target_samples, alg="snpe_c")
@@ -167,7 +217,7 @@ def test_c2st_sre_on_linearGaussian(
     # Should use default `num_atoms=10` for SRE; `num_atoms=2` for AALR
     posterior = infer(num_rounds=1, num_simulations_per_round=1000).set_default_x(x_o)
 
-    samples = posterior.sample(sample_shape=(num_samples,), thin=3)
+    samples = posterior.sample(sample_shape=(num_samples,), mcmc_parameters = {"thin": 3})
 
     # Check performance based on c2st accuracy.
     check_c2st(samples, target_samples, alg=f"sre-{prior_str}-{method_str}")
